@@ -13,6 +13,7 @@ import zipfile
 
 # Internal Python files
 from .utils import get_current_time_in_utc, remove_dir
+from .xliff import XLIFF
 
 nsmap = {
     'kaplan': 'https://kaplan.pro',
@@ -20,123 +21,21 @@ nsmap = {
     'xml': 'http://www.w3.org/XML/1998/namespace'
 }
 
-class KXLIFF:
+class KXLIFF(XLIFF):
     '''
     A slightly modified version of the XML Localisation File Format (http://docs.oasis-open.org/xliff/xliff-core/v2.1/xliff-core-v2.1.html).
 
     Class KXLIFF offers native support for .xliff and .sdlxliff files.
 
     Args:
-        bilingual_file: Either the path to a bilingual file or a bilingual file as a BytesIO instance.
-        src (optional): ISO 639-1 code for the source language.
-        trgt (optional): ISO 639-1 code for the target language.
+        name: Name of the file
+        xml_root: The xml root of the file
     '''
 
-    def __init__(self, bilingual_file, src=None, trgt=None):
-        self.name = os.path.basename(bilingual_file) if type(bilingual_file) == str else bilingual_file.name
-        self.nsmap = {}
-        self.translation_units = etree.Element('translation-units')
-        self.xliff_variant = ''
-        self.xml_root = etree.parse(bilingual_file).getroot()
-
-        self.xliff_version = float(self.xml_root.attrib['version'])
-        self.nsmap = self.xml_root.nsmap
-
-        if self.name.lower().endswith('.kxliff'):
-            self.xliff_variant = 'kaplan'
-        elif self.name.lower().endswith('.sdlxliff'):
-            self.xliff_variant = 'sdl'
-        elif self.name.lower().endswith('.xliff'):
-            self.xliff_variant = 'vanilla'
-        else:
-            raise ValueError('File is incompatible!')
-
-        if self.xliff_version >= 2.0:
-            for translation_unit in self.xml_root.findall('.//unit', self.nsmap):
-                _translation_unit = deepcopy(translation_unit)
-                _translation_unit.tag = 'translation-unit'
-                for _child in _translation_unit:
-                    if not _child.tag.endswith(('}segment', '}ignorable')):
-                        _translation_unit.remove(_child)
-                for _any_child in _translation_unit.findall('.//'):
-                    if 'equiv' in _any_child.attrib:
-                        _any_child.text = html.unescape(_any_child.attrib['equiv'])
-
-                self.translation_units.append(_translation_unit)
-        else:
-            for translation_unit in self.xml_root.findall('.//trans-unit', self.nsmap):
-                segments = []
-                _g_counter = {}
-                _translation_unit = etree.Element('translation-unit', translation_unit.attrib)
-                if translation_unit.find('seg-source', self.nsmap) is not None:
-                    for source_segment in translation_unit.findall('seg-source//mrk', self.nsmap):
-                        target_segment = translation_unit.find('target//mrk[@mid="{0}"]'.format(source_segment.attrib['mid']), self.nsmap)
-
-                        segments.append([source_segment, target_segment])
-
-                else:
-                    segments.append([translation_unit.find('source', self.nsmap), translation_unit.find('target', self.nsmap)])
-
-                for segment in segments:
-                    _segment = etree.SubElement(_translation_unit, 'segment', {'id': segment[0].attrib.get('mid', 'N/A')})
-
-                    _source = deepcopy(segment[0])
-                    _source.tag = 'source'
-                    _source.tail = None
-
-                    if segment[1] is not None:
-                        _target = deepcopy(segment[1])
-                    else:
-                        _target = etree.Element('mrk', _source.attrib)
-                    _target.tag = 'target'
-                    _target.tail = None
-
-                    _segment.append(_source)
-                    _segment.append(_target)
-
-                    for _child in _segment:
-                        for _any_child in _child.findall('.//'):
-
-                            if _any_child.tag.startswith('b'):
-                                _any_child.text = '<{0}-{1}>'.format(etree.QName(_any_child).localname[1:], _any_child.attrib.get('id', 'N/A'))
-                            elif _any_child.tag.startswith('e'):
-                                _any_child.text = '</{0}-{1}>'.format(etree.QName(_any_child).localname[1:], _any_child.attrib.get('id', 'N/A'))
-                            elif _any_child.tag.endswith('g'):
-                                if _any_child.attrib['id'] not in _g_counter:
-                                    _g_counter[_any_child.attrib['id']] = str(len(_g_counter) + 1)
-                                _b_g_tag = etree.Element('g', _any_child.attrib)
-                                _b_g_tag.attrib['no'] = _g_counter[_any_child.attrib['id']]
-                                _e_g_tag = deepcopy(_b_g_tag)
-
-                                _b_g_tag.text = '<g-{0}>'.format(_g_counter[_any_child.attrib['id']])
-                                _e_g_tag.text = '</g-{0}>'.format(_g_counter[_any_child.attrib['id']])
-
-                                _parent = _any_child.getparent()
-                                _parent.replace(_any_child, _b_g_tag)
-                                _b_g_tag.tail = _any_child.text
-                                next_i = _parent.index(_b_g_tag) + 1
-                                for _g_child in _any_child:
-                                    _parent.insert(next_i, _g_child)
-                                    next_i += 1
-                                _parent.insert(next_i, _e_g_tag)
-                                _e_g_tag.tail = _any_child.tail
-                            else:
-                                _any_child.text = '<{0}-{1}/>'.format(etree.QName(_any_child).localname, _any_child.attrib.get('id', 'N/A'))
-
-                self.translation_units.append(_translation_unit)
-
-    @staticmethod
-    def can_process(source_file):
-        '''
-        Static method that determines whether the source_file is compatible.
-
-        Args:
-            source_file: Path to a source file.
-        '''
-        if source_file.lower().endswith(('.docx', '.kxliff', '.odp', '.ods', '.odt', '.po', '.sdlxliff', '.txt', '.xliff')):
-            return True
-        else:
-            return False
+    def __init__(self, name, xml_root):
+        if not name.lower().endswith('.kxliff') or 'kaplan' not in xml_root.nsmap:
+            raise TypeError('This class may only handle .kxliff files.')
+        super().__init__(name, xml_root)
 
     def generate_target_translation(self, output_directory, path_to_source_file=None):
         '''
@@ -481,8 +380,6 @@ class KXLIFF:
             list_of_segments: List containing segment IDs.
         '''
 
-        assert self.xliff_variant == 'kaplan', 'This function is available for .kxliff files only.'
-
         def transfer_children(source_parent, target_parent):
             if source_parent.text is not None:
                 if len(target_parent) == 0:
@@ -1281,165 +1178,4 @@ class KXLIFF:
             segment.attrib['id'] = str(segment_counter)
             segment_counter += 1
 
-        kxliff = BytesIO(etree.tostring(xml_root, encoding='UTF-8'))
-        kxliff.name = name + '.kxliff'
-
-        return cls(kxliff)
-
-    def save(self, output_directory):
-        self.xml_root.getroottree().write(os.path.join(output_directory, self.name),
-                                          encoding='UTF-8',
-                                          xml_declaration=True)
-
-    def update_segment(self, segment_state, target_segment, tu_no, segment_no=None):
-        '''
-        Updates a given segment.
-
-        Args:
-            segment_state (str): The state of the segment (ie. translated, signed-off, etc.).
-            target_segment (str): Target segment in HTML.
-            tu_no (str or int): The number of the translation unit .
-            segment_no (str or int) (optional): The number of the segment. Segments
-                                                that make up the entire tu do not have numbers.
-        '''
-
-        target_segment = etree.fromstring(target_segment)
-
-        assert etree.QName(target_segment).localname == 'target'
-
-        for any_child in target_segment:
-            if 'dataref' in any_child.attrib:
-                any_child.attrib['dataRef'] = any_child.attrib.pop('dataref')
-
-            any_child.attrib.pop('contenteditable', None)
-
-        if self.xliff_version < 2.0:
-            active_g_tags = []
-            for any_child in target_segment:
-                if any_child.tag.endswith('g'):
-                    if any_child.tail is not None:
-                        if len(active_g_tags) > 0:
-                            active_g_tag = active_g_tags[0]
-                            if len(active_g_tag) > 0:
-                                if active_g_tag[-1].tail is None:
-                                    active_g_tag[-1].tail = ''
-                                active_g_tag[-1].tail += any_child.tail
-                            else:
-                                if active_g_tag.text is None:
-                                    active_g_tag.text = ''
-                                active_g_tag.text += any_child.tail
-
-                            if any_child.text.startswith('</'):
-                                for g_tag in active_g_tags:
-                                    if g_tag.attrib['id'] == any_child.attrib['id']:
-                                        active_g_tags.remove(g_tag)
-                                        if len(active_g_tags) > 0:
-                                            target_segment.replace(any_child, active_g_tags[0])
-                                        else:
-                                            target_segment.remove(any_child)
-                                        break
-                            else:
-                                any_child.text = None
-                                any_child.tail = None
-                                active_g_tags.append(any_child)
-                                target_segment.remove(any_child)
-
-                        else:
-                            if any_child.text.startswith('</'):
-                                preceding_sibling = any_child.getprevious()
-                                if preceding_sibling is not None:
-                                    if target_segment.text is None:
-                                        target_segment.text = ''
-                                    target_segment.text += any_child.tail
-                                else:
-                                    if preceding_sibling.tail is None:
-                                        preceding_sibling.tail = ''
-                                    preceding_sibling.tail += any_child.tail
-                                target_segment.remove(any_child)
-                            else:
-                                any_child.text = any_child.tail
-                                any_child.tail = None
-                                active_g_tags.append(any_child)
-
-                    else:
-                        if any_child.text.startswith('</'):
-                            for g_tag in active_g_tags:
-                                if g_tag.attrib['id'] == any_child.attrib['id']:
-                                    active_g_tags.remove(g_tag)
-                                    if len(active_g_tags) > 0:
-                                        target_segment.replace(any_child, active_g_tags[0])
-                                    else:
-                                        target_segment.remove(any_child)
-                                    break
-                        else:
-                            any_child.text = None
-                            if len(active_g_tags) > 0:
-                                target_segment.remove(any_child)
-                            active_g_tags.append(any_child)
-
-                else:
-                    any_child.text = None
-                    if len(active_g_tags) > 0:
-                        active_g_tags[0].append(any_child)
-
-        else:
-            target_segment.tag = '{{{0}}}target'.format(self.nsmap[None])
-            for child in target_segment:
-                child.tag = '{{{0}}}{1}'.format(self.nsmap[None], etree.QName(child).localname)
-
-        _target_segment = deepcopy(target_segment)
-
-        translation_unit = self.translation_units.find('translation-unit[@id="{0}"]'.format(tu_no))
-
-        if segment_no is not None:
-            segment = translation_unit.find('segment[@id="{0}"]'.format(segment_no), self.nsmap)
-            if segment is None:
-                segment = translation_unit.find('segment[@id="{0}"]'.format(segment_no))
-        else:
-            segment = translation_unit.findall('segment', self.nsmap)[0]
-            if segment is None:
-                segment = translation_unit.findall('segment')[0]
-
-        target = segment.find('target', self.nsmap)
-        if target is None:
-            target = segment.find('target')
-        if target is None:
-            segment.append(target_segment)
-        else:
-            segment[segment.index(target)] = target_segment
-
-        if self.xliff_version >= 2.0:
-            _translation_unit = self.xml_root.find('.//unit[@id="{0}"]'.format(tu_no), self.nsmap)
-
-            if segment_no is not None:
-                _segment = _translation_unit.find('segment[@id="{0}"]'.format(segment_no), self.nsmap)
-            else:
-                _segment = _translation_unit.findall('segment', self.nsmap)[0]
-
-            _segment.attrib['state'] = segment_state
-
-            for any_child in _target_segment:
-                any_child.text = None
-
-            _target = _segment.find('target', self.nsmap)
-            if _target is None:
-                _target = _segment.find('target')
-            if _target is None:
-                _segment.append(_target_segment)
-            else:
-                _segment[_segment.index(_target)] = _target_segment
-
-        else:
-            _translation_unit = self.xml_root.find('.//trans-unit[@id="{0}"]'.format(tu_no), self.nsmap)
-            _target_segment.attrib['state'] = segment_state
-
-            if segment_no is not None:
-                _segment = _translation_unit.find('target//mrk[@mid="{0}"]'.format(segment_no), self.nsmap)
-                _target_segment.tag = '{{{0}}}mrk'.format(self.nsmap[None])
-                _target_segment.attrib['mtype'] = 'seg'
-                _target_segment.attrib['mid'] = str(segment_no)
-            else:
-                _segment = _translation_unit.find('target', self.nsmap)
-                _target_segment.tag = '{{{0}}}target'.format(self.nsmap[None])
-
-            _segment.getparent().replace(_segment, _target_segment)
+        return cls(name + '.kxliff', xml_root)
